from math import *

from abqpy import run

run(cae=False)

from abaqus.Odb.OdbCommands import *
from abaqus.UtilityAndView.BackwardCompatibility import BackwardCompatibility
from abaqusConstants import *

backwardCompatibility = BackwardCompatibility()
